unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVStyle, RVScroll, RichView, StdCtrls, CRVData, RVTypes, RVItem,
  RVDocX, RVStrFuncs;

type

  TForm1 = class(TForm)
    RichView1: TRichView;
    RVStyle1: TRVStyle;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

// A4 width in twips (1 inch = 1440 twips)
const A4Width = 11906;
      A4Height = 16838;

      rvsSectionBreak = -2000;

type

  TRVSectionBreakItemInfo = class (TRVFullLineItemInfo)
  private
    FPageWidthTw, FPageHeightTw: Integer;
    FLandscape, FFinal: Boolean;
  public
    constructor CreateEx(ARVData: TPersistent; APageWidthTw, APageHeightTw: Integer;
      ALandscape, AFinal: Boolean);
    function GetBoolValue(Prop: TRVItemBoolProperty): Boolean; override;
    procedure SaveOOXML(Stream: TStream; ARVData: TPersistent; ItemNo: Integer;
      SavingData: TRVDocXSavingData; HiddenParent: Boolean); override;
  end;

{=========================== TRVSectionBreakItemInfo ==========================}
constructor TRVSectionBreakItemInfo.CreateEx(ARVData: TPersistent;
  APageWidthTw, APageHeightTw: Integer; ALandscape, AFinal: Boolean);
begin
  inherited Create(ARVData);
  StyleNo := rvsSectionBreak;
  FPageWidthTw := APageWidthTw;
  FPageHeightTw := APageHeightTw;
  FFinal := AFinal;
end;

function TRVSectionBreakItemInfo.GetBoolValue(
  Prop: TRVItemBoolProperty): Boolean;
begin
  case Prop of
    rvbpDocXInRun:
      Result := False; // saving not inside a paragraph
    else
      Result := inherited GetBoolValue(Prop);
  end;
end;

procedure TRVSectionBreakItemInfo.SaveOOXML(Stream: TStream;
  ARVData: TPersistent; ItemNo: Integer; SavingData: TRVDocXSavingData;
  HiddenParent: Boolean);
var
  s, ors: AnsiString;

begin
  if FLandscape then
    ors := ' w:orient="landscape"'
  else
    ors := '';
  s := RVFormatA(
    '<w:sectPr><w:pgSz w:w="%d" w:h="%d"%s/></w:sectPr>',
    [FPageWidthTw, FPageHeightTw, ors]);
  if not FFinal then
    s := '<w:p><w:pPr>' + s + '</w:pPr></w:p>';
  RVWrite(Stream, s);
end;
{==============================================================================}
procedure CreateDocXWithSections(const RVFFiles: array of String;
  const IsLandscape: array of Boolean;
  const DocXFile: String; rv: TCustomRichView);
var
  i: Integer;
  Stream: TFileStream;
  SectionBreak: TRVSectionBreakItemInfo;
begin
  rv.Clear;
  rv.DeleteUnusedStyles(True, True, True);
  for i := Low(RVFFiles) to High(RVFFiles) do
  begin
    try
      Stream := TFileStream.Create(RVFFiles[i], fmOpenRead);
      try
        rv.InsertRVFFromStream(Stream, rv.ItemCount);
      finally
        Stream.Free;
      end;
    except
      rv.AddNL('Error loading file', 0, 0);
    end;
    if IsLandscape[i] then
      SectionBreak := TRVSectionBreakItemInfo.CreateEx(rv.RVData, A4Height, A4Width,
        True, i = High(RVFFiles))
    else
      SectionBreak := TRVSectionBreakItemInfo.CreateEx(rv.RVData, A4Width, A4Height,
        False, i = High(RVFFiles));
    rv.AddItem('', SectionBreak);
  end;
  rv.Format;
  rv.SaveDocX(DocXFile, False);
end;
{============================== TForm1 ========================================}
procedure TForm1.Button1Click(Sender: TObject);
begin
  CreateDocXWithSections(
    ['doc1.rvf', 'doc2.rvf', 'doc3.rvf'],
    [False, True, False], 'Result.docx', RichView1);
  RichView1.Format;
end;

end.
